;**********************************************************************
;    Filename:	    dengen2.asm                                           *
;    Date:          2002/7/7                                         *
;    File Version:                                                    *
;**********************************************************************

	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;***** VARIABLE DEFINITIONS
w_temp        EQU     0x0C        ; variable used for context saving 
status_temp   EQU     0x0D        ; variable used for context saving


wait_data	EQU	0x15		;wait data ram

taihi		EQU	0x17		;taihi data
suji_1		EQU	0x18		;suji (10)
suji_2		EQU	0x19		;suji (1)

d_count		EQU	0x1d		;hyouji count data
count		EQU	0x1e		;warikomi count data


tm0_setdata	EQU	0xdf		;FFH-DFH=32 32*2=64usec
;**********************************************************************
		ORG     0x000             ; processor reset vector
  		goto    main              ; go to beginning of program


		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		;---------------------------------------------------------------
		bcf	INTCON,T0IF
		movlw	tm0_setdata
		movwf	TMR0

		incf	count,1
		;---------------------------------------------------------------
int_end
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt
;--------------------------------------------------------------------------
main
		call	port_set
		movlw	0bh
		movwf	PORTA
		movlw	00h
		movwf	PORTB

		call	set_dcount

		call	timer0_set

		movlw	00h
		movwf	suji_1
		movwf	suji_2

;--------------------------------------
main_loop
		decf	d_count,1
		btfss	STATUS,2
		goto	main_led
		call	set_dcount
		call	keisoku

		call	denatu_henkan
		movwf	count

		call	suji_henkan

		movf	suji_1,0
		call	data_henkan
		movwf	suji_1

		movf	suji_2,0
		call	data_henkan
		movwf	suji_2

main_led

		movf	suji_2,0
		movwf	PORTB
		bcf	PORTA,1
		nop
		call	wait_0
		bsf	PORTA,1
		nop
		movf	suji_1,0
		movwf	PORTB
		bcf	PORTA,0
		nop
		call	wait_0
		bsf	PORTA,0
		nop

;---------------------------------------
		goto	main_loop
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
wait_0
		movlw	0ffh
		movwf	wait_data
		goto	wait_2
wait_1		
		movlw	0ffh
		movwf	wait_data
wait_2
		nop
		decfsz	wait_data,1
		goto	wait_2
		return
;-------------------------------------------------------------------------------
keisoku
		movlw	00h
		movwf	count
		call	ra2_out
		bcf	PORTA,2
		nop
		call	wait_1
		call	ra2_in
		bsf	INTCON,T0IE
keisoku2	nop
		btfsc	PORTA,3
		goto	keisoku2
		bcf	INTCON,T0IE
		return	
;------------------------------------------------------------------------------
set_dcount
		movlw	0x80
		movwf	d_count
		return
;-------------------------------------------------------------------------------
ra2_out
					;RA2(OUT)
		bsf	STATUS,RP0
		bcf	TRISA,2
		bcf	STATUS,RP0
		return
;--------------------------------------------------------------------------------
ra2_in
					;RA2(IN)
		bsf	STATUS,RP0
		bsf	TRISA,2
		bcf	STATUS,RP0
		return
;--------------------------------------------------------------------------------
suji_henkan
		movf	count,0
		andlw	0fh
		movwf	suji_2
		swapf	count,0
		andlw	0fh
		movwf	suji_1
		return
;------------------------------------------------------------------------------
data_henkan	
		movwf	taihi
		movlw	low(data_table)
		addwf	taihi,1
		movlw	high(data_table)
		movwf	PCLATH
		addcf	PCLATH,1
		movf	taihi,0
		movwf	PCL
data_table	;----------- 
		retlw	05f		;0
		retlw	00c		;1
		retlw	03b		;2
		retlw	03e		;3
		retlw	06c		;4
		retlw	076		;5
		retlw	077		;6
		retlw	05c		;7
		retlw	07f		;8
		retlw	07e		;9
		retlw	07d		;A
		retlw	067		;B
		retlw	053		;C
		retlw	02f		;D
		retlw	020		;E(-)
		retlw	0df		;F(GREEN+0)

;------------------------------------------------------------------------------
denatu_henkan	movf	count,0
		movwf	taihi
		movlw	low(denatu_table)
		addwf	taihi,1
		movlw	high(denatu_table)
		movwf	PCLATH
		addcf	PCLATH,1
		movf	taihi,0
		movwf	PCL
denatu_table	;----------- 
		retlw	0ee		;00
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	010
		retlw	011
		retlw	012
		retlw	012
		retlw	013		;10
		retlw	014
		retlw	015
		retlw	015
		retlw	016
		retlw	017
		retlw	018
		retlw	018
		retlw	019
		retlw	020
		retlw	021
		retlw	021
		retlw	022
		retlw	023
		retlw	024
		retlw	024
		retlw	025		;20
		retlw	026
		retlw	027
		retlw	027
		retlw	028
		retlw	029
		retlw	029
		retlw	030
		retlw	031
		retlw	032
		retlw	032
		retlw	033
		retlw	034
		retlw	034
		retlw	035
		retlw	036
		retlw	037		;30
		retlw	037
		retlw	038
		retlw	039
		retlw	039
		retlw	040
		retlw	041
		retlw	041
		retlw	042
		retlw	043
		retlw	043
		retlw	044
		retlw	045
		retlw	045
		retlw	046
		retlw	047
		retlw	047		;40
		retlw	048
		retlw	049
		retlw	049
		retlw	050
		retlw	051
		retlw	051
		retlw	052
		retlw	052
		retlw	053
		retlw	054
		retlw	054
		retlw	055
		retlw	056
		retlw	056
		retlw	057
		retlw	058		;50
		retlw	058
		retlw	059
		retlw	059
		retlw	060
		retlw	061
		retlw	061
		retlw	062
		retlw	062
		retlw	063
		retlw	064
		retlw	064
		retlw	065
		retlw	065
		retlw	066
		retlw	067
		retlw	067		;60
		retlw	068
		retlw	068
		retlw	069
		retlw	069
		retlw	070
		retlw	071
		retlw	071
		retlw	072
		retlw	072
		retlw	073
		retlw	073
		retlw	074
		retlw	075
		retlw	075
		retlw	076
		retlw	076		;70
		retlw	077
		retlw	077
		retlw	078
		retlw	078
		retlw	079
		retlw	080
		retlw	080
		retlw	081
		retlw	081
		retlw	082
		retlw	082
		retlw	083
		retlw	083
		retlw	084
		retlw	084
		retlw	085		;80
		retlw	085
		retlw	086
		retlw	086
		retlw	087
		retlw	087
		retlw	088
		retlw	089
		retlw	089
		retlw	090
		retlw	090
		retlw	091
		retlw	091
		retlw	092
		retlw	092
		retlw	093
		retlw	093		;90
		retlw	094
		retlw	094
		retlw	095
		retlw	095
		retlw	096
		retlw	096
		retlw	096
		retlw	097
		retlw	097
		retlw	098
		retlw	098
		retlw	099
		retlw	099
		retlw	0f0
		retlw	0f0
		retlw	0f1		;A0
		retlw	0f1
		retlw	0f2
		retlw	0f2
		retlw	0f3
		retlw	0f3
		retlw	0f4
		retlw	0f4
		retlw	0f4
		retlw	0f5
		retlw	0f5
		retlw	0f6
		retlw	0f6
		retlw	0f7
		retlw	0f7
		retlw	0f8
		retlw	0f8		;B0
		retlw	0f8
		retlw	0f9
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee
		retlw	0ee

;-------------------------------------------------------------------------------
port_set				;RA0,RA1,RA2,RA4(OUT)
					;RA3(IN)
					;RB0,RB1,RB2,RB3,RB4,RB5,RB6,RA7(OUT)
		bsf	STATUS,RP0
		clrf	TRISA
		bsf	TRISA,3
		nop
		clrf	TRISB
		bcf	STATUS,RP0
		return

timer0_set

		bsf	STATUS,RP0
		movlw	080h		;(1)PORTB NOT PULLUP
					;(0)INTEDG
					;(0)TOCS
					;(0)TOSE
					;(0)PSA
					;(000)1/2
		movwf	OPTION_REG
		bcf	STATUS,RP0

		movlw	tm0_setdata
		movwf	TMR0
		bsf	INTCON,T0IE
		bsf	INTCON,GIE
		return


		END                     ; directive 'end of program'

